from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="Editor")


@_attrs_define
class Editor:
    """
    Attributes:
        affiliation (str): Primary affiliation of the editor.
        name (str): Name of the editor.
        role (str): Role of the editor in the context of the topic.
        description (str): Description of the editor's focus, concerns, and motives.
    """

    affiliation: str
    name: str
    role: str
    description: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        affiliation = self.affiliation

        name = self.name

        role = self.role

        description = self.description

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "affiliation": affiliation,
                "name": name,
                "role": role,
                "description": description,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        affiliation = d.pop("affiliation")

        name = d.pop("name")

        role = d.pop("role")

        description = d.pop("description")

        editor = cls(
            affiliation=affiliation,
            name=name,
            role=role,
            description=description,
        )

        editor.additional_properties = d
        return editor

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.chat_message import ChatMessage


T = TypeVar("T", bound="ChatInputReq")


@_attrs_define
class ChatInputReq:
    """
    Attributes:
        messages (List['ChatMessage']):
        chat_id (Union[None, Unset, str]):
    """

    messages: List["ChatMessage"]
    chat_id: Union[None, Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        messages = []
        for messages_item_data in self.messages:
            messages_item = messages_item_data.to_dict()
            messages.append(messages_item)

        chat_id: Union[None, Unset, str]
        if isinstance(self.chat_id, Unset):
            chat_id = UNSET
        else:
            chat_id = self.chat_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "messages": messages,
            }
        )
        if chat_id is not UNSET:
            field_dict["chat_id"] = chat_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.chat_message import ChatMessage

        d = src_dict.copy()
        messages = []
        _messages = d.pop("messages")
        for messages_item_data in _messages:
            messages_item = ChatMessage.from_dict(messages_item_data)

            messages.append(messages_item)

        def _parse_chat_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        chat_id = _parse_chat_id(d.pop("chat_id", UNSET))

        chat_input_req = cls(
            messages=messages,
            chat_id=chat_id,
        )

        chat_input_req.additional_properties = d
        return chat_input_req

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains all the data models used in inputs/outputs"""

from .agent import Agent
from .agent_public import AgentPublic
from .agents_public import AgentsPublic
from .blog_post_create_req import BlogPostCreateReq
from .blog_post_create_res import BlogPostCreateRes
from .blog_post_detail_res import BlogPostDetailRes
from .blog_post_update_req import BlogPostUpdateReq
from .blog_post_update_res import BlogPostUpdateRes
from .blog_writer_agent_state import BlogWriterAgentState
from .blog_writer_agent_state_messages_type_0_item import BlogWriterAgentStateMessagesType0Item
from .body_auth_login_access_token import BodyAuthLoginAccessToken
from .chat_input import ChatInput
from .chat_input_req import ChatInputReq
from .chat_message import ChatMessage
from .config_response import ConfigResponse
from .doc_coll_create import DocCollCreate
from .doc_coll_public import DocCollPublic
from .doc_colls_public import DocCollsPublic
from .editor import Editor
from .hello_api_public import HelloApiPublic
from .http_validation_error import HTTPValidationError
from .interview_state import InterviewState
from .interview_state_references_type_0 import InterviewStateReferencesType0
from .item_create import ItemCreate
from .item_public import ItemPublic
from .item_update import ItemUpdate
from .items_public import ItemsPublic
from .joke_agent_state import JokeAgentState
from .joke_agent_state_messages_type_0_item import JokeAgentStateMessagesType0Item
from .message import Message
from .message_ack_request import MessageAckRequest
from .message_public import MessagePublic
from .message_public_message import MessagePublicMessage
from .message_pull_item import MessagePullItem
from .message_pull_req import MessagePullReq
from .message_pull_response import MessagePullResponse
from .message_pull_response_item import MessagePullResponseItem
from .message_send_public import MessageSendPublic
from .message_send_public_messages_item import MessageSendPublicMessagesItem
from .new_password import NewPassword
from .outline import Outline
from .post import Post
from .rag_retrieval_req import RagRetrievalReq
from .read_file_req import ReadFileReq
from .research_state import ResearchState
from .run_bash_req import RunBashReq
from .runnable_config import RunnableConfig
from .runnable_config_configurable import RunnableConfigConfigurable
from .runnable_config_metadata import RunnableConfigMetadata
from .section import Section
from .sub_app import SubApp
from .sub_web import SubWeb
from .subsection import Subsection
from .test_url_req import TestUrlReq
from .token import Token
from .update_password import UpdatePassword
from .user_create import UserCreate
from .user_public import UserPublic
from .user_register import UserRegister
from .user_update import UserUpdate
from .user_update_me import UserUpdateMe
from .users_public import UsersPublic
from .validation_error import ValidationError
from .wiki_section import WikiSection
from .workspace import Workspace

__all__ = (
    "Agent",
    "AgentPublic",
    "AgentsPublic",
    "BlogPostCreateReq",
    "BlogPostCreateRes",
    "BlogPostDetailRes",
    "BlogPostUpdateReq",
    "BlogPostUpdateRes",
    "BlogWriterAgentState",
    "BlogWriterAgentStateMessagesType0Item",
    "BodyAuthLoginAccessToken",
    "ChatInput",
    "ChatInputReq",
    "ChatMessage",
    "ConfigResponse",
    "DocCollCreate",
    "DocCollPublic",
    "DocCollsPublic",
    "Editor",
    "HelloApiPublic",
    "HTTPValidationError",
    "InterviewState",
    "InterviewStateReferencesType0",
    "ItemCreate",
    "ItemPublic",
    "ItemsPublic",
    "ItemUpdate",
    "JokeAgentState",
    "JokeAgentStateMessagesType0Item",
    "Message",
    "MessageAckRequest",
    "MessagePublic",
    "MessagePublicMessage",
    "MessagePullItem",
    "MessagePullReq",
    "MessagePullResponse",
    "MessagePullResponseItem",
    "MessageSendPublic",
    "MessageSendPublicMessagesItem",
    "NewPassword",
    "Outline",
    "Post",
    "RagRetrievalReq",
    "ReadFileReq",
    "ResearchState",
    "RunBashReq",
    "RunnableConfig",
    "RunnableConfigConfigurable",
    "RunnableConfigMetadata",
    "Section",
    "SubApp",
    "Subsection",
    "SubWeb",
    "TestUrlReq",
    "Token",
    "UpdatePassword",
    "UserCreate",
    "UserPublic",
    "UserRegister",
    "UsersPublic",
    "UserUpdate",
    "UserUpdateMe",
    "ValidationError",
    "WikiSection",
    "Workspace",
)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.editor import Editor
    from ..models.interview_state import InterviewState
    from ..models.outline import Outline
    from ..models.wiki_section import WikiSection


T = TypeVar("T", bound="ResearchState")


@_attrs_define
class ResearchState:
    """
    Attributes:
        topic (str):
        outline (Outline):
        editors (List['Editor']):
        interview_results (List['InterviewState']):
        sections (List['WikiSection']):
        article (str):
    """

    topic: str
    outline: "Outline"
    editors: List["Editor"]
    interview_results: List["InterviewState"]
    sections: List["WikiSection"]
    article: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        topic = self.topic

        outline = self.outline.to_dict()

        editors = []
        for editors_item_data in self.editors:
            editors_item = editors_item_data.to_dict()
            editors.append(editors_item)

        interview_results = []
        for interview_results_item_data in self.interview_results:
            interview_results_item = interview_results_item_data.to_dict()
            interview_results.append(interview_results_item)

        sections = []
        for sections_item_data in self.sections:
            sections_item = sections_item_data.to_dict()
            sections.append(sections_item)

        article = self.article

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "topic": topic,
                "outline": outline,
                "editors": editors,
                "interview_results": interview_results,
                "sections": sections,
                "article": article,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.editor import Editor
        from ..models.interview_state import InterviewState
        from ..models.outline import Outline
        from ..models.wiki_section import WikiSection

        d = src_dict.copy()
        topic = d.pop("topic")

        outline = Outline.from_dict(d.pop("outline"))

        editors = []
        _editors = d.pop("editors")
        for editors_item_data in _editors:
            editors_item = Editor.from_dict(editors_item_data)

            editors.append(editors_item)

        interview_results = []
        _interview_results = d.pop("interview_results")
        for interview_results_item_data in _interview_results:
            interview_results_item = InterviewState.from_dict(interview_results_item_data)

            interview_results.append(interview_results_item)

        sections = []
        _sections = d.pop("sections")
        for sections_item_data in _sections:
            sections_item = WikiSection.from_dict(sections_item_data)

            sections.append(sections_item)

        article = d.pop("article")

        research_state = cls(
            topic=topic,
            outline=outline,
            editors=editors,
            interview_results=interview_results,
            sections=sections,
            article=article,
        )

        research_state.additional_properties = d
        return research_state

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

if TYPE_CHECKING:
    from ..models.message_public_message import MessagePublicMessage


T = TypeVar("T", bound="MessagePublic")


@_attrs_define
class MessagePublic:
    """
    Attributes:
        msg_id (int):
        read_ct (int):
        enqueued_at (datetime.datetime):
        vt (datetime.datetime):
        message (MessagePublicMessage):
    """

    msg_id: int
    read_ct: int
    enqueued_at: datetime.datetime
    vt: datetime.datetime
    message: "MessagePublicMessage"
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        msg_id = self.msg_id

        read_ct = self.read_ct

        enqueued_at = self.enqueued_at.isoformat()

        vt = self.vt.isoformat()

        message = self.message.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "msg_id": msg_id,
                "read_ct": read_ct,
                "enqueued_at": enqueued_at,
                "vt": vt,
                "message": message,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.message_public_message import MessagePublicMessage

        d = src_dict.copy()
        msg_id = d.pop("msg_id")

        read_ct = d.pop("read_ct")

        enqueued_at = isoparse(d.pop("enqueued_at"))

        vt = isoparse(d.pop("vt"))

        message = MessagePublicMessage.from_dict(d.pop("message"))

        message_public = cls(
            msg_id=msg_id,
            read_ct=read_ct,
            enqueued_at=enqueued_at,
            vt=vt,
            message=message,
        )

        message_public.additional_properties = d
        return message_public

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="ChatMessage")


@_attrs_define
class ChatMessage:
    """
    Attributes:
        id (Union[Unset, str]):
        content (Union[None, Unset, str]):
        created_at (Union[Unset, datetime.datetime]):
        updated_at (Union[Unset, datetime.datetime]):
        chat_id (Union[None, Unset, str]):
        role (Union[None, Unset, str]):  Default: 'user'.
    """

    id: Union[Unset, str] = UNSET
    content: Union[None, Unset, str] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    chat_id: Union[None, Unset, str] = UNSET
    role: Union[None, Unset, str] = "user"
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        content: Union[None, Unset, str]
        if isinstance(self.content, Unset):
            content = UNSET
        else:
            content = self.content

        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        chat_id: Union[None, Unset, str]
        if isinstance(self.chat_id, Unset):
            chat_id = UNSET
        else:
            chat_id = self.chat_id

        role: Union[None, Unset, str]
        if isinstance(self.role, Unset):
            role = UNSET
        else:
            role = self.role

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if content is not UNSET:
            field_dict["content"] = content
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if chat_id is not UNSET:
            field_dict["chat_id"] = chat_id
        if role is not UNSET:
            field_dict["role"] = role

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        def _parse_content(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        content = _parse_content(d.pop("content", UNSET))

        _created_at = d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updated_at", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        def _parse_chat_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        chat_id = _parse_chat_id(d.pop("chat_id", UNSET))

        def _parse_role(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        role = _parse_role(d.pop("role", UNSET))

        chat_message = cls(
            id=id,
            content=content,
            created_at=created_at,
            updated_at=updated_at,
            chat_id=chat_id,
            role=role,
        )

        chat_message.additional_properties = d
        return chat_message

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
